@@ -5,9 +5,8 @@
 #include "selinux_internal.h"
 #include "context_internal.h"
 
-int rpm_execcon(unsigned int verified __attribute__((unused)), 
-		const char *filename, 
-		char *const argv[], char *const envp[])
+int rpm_execcon(unsigned int verified __attribute__ ((unused)),
+		const char *filename, char *const argv[], char *const envp[])
 {
 	security_context_t mycon = NULL, fcon = NULL, newcon = NULL;
 	context_t con = NULL;
@@ -44,12 +43,12 @@ int rpm_execcon(unsigned int verified __attribute__((unused)),
 	}
 
 	rc = setexeccon(newcon);
-	if (rc < 0) 
+	if (rc < 0)
 		goto out;
-out:
+      out:
 
 	if (rc >= 0 || security_getenforce() < 1)
-		rc = execve(filename, argv, envp); 
+		rc = execve(filename, argv, envp);
 
 	context_free(con);
 	freecon(newcon);
