@@ -9,8 +9,20 @@
 
 int setfilecon_raw(const char *path, const security_context_t context)
 {
-	return setxattr(path, XATTR_NAME_SELINUX, context, strlen(context) + 1,
+	int rc = setxattr(path, XATTR_NAME_SELINUX, context, strlen(context) + 1,
 			0);
+	if (rc < 0 && errno == ENOTSUP) {
+		security_context_t ccontext = NULL;
+		int err = errno;
+		if ((getfilecon_raw(path, &ccontext) >= 0) &&
+		    (strcmp(context,ccontext) == 0)) {
+			rc = 0;
+		} else {
+			errno = err;
+		}
+		freecon(ccontext);
+	}
+	return rc;
 }
 
 hidden_def(setfilecon_raw)
