@@ -6,7 +6,7 @@
 #include <string.h>
 #include <selinux/selinux.h>
 
-void usage(const char *progname)
+static void usage(const char *progname)
 {
 	fprintf(stderr, "usage:  %s -a or %s boolean...\n", progname, progname);
 	exit(1);
@@ -14,7 +14,7 @@ void usage(const char *progname)
 
 int main(int argc, char **argv)
 {
-	int i, rc = 0, active, pending, len = 0, opt;
+	int i, get_all = 0, rc = 0, active, pending, len = 0, opt;
 	char **names;
 
 	while ((opt = getopt(argc, argv, "a")) > 0) {
@@ -39,6 +39,7 @@ int main(int argc, char **argv)
 				printf("No booleans\n");
 				return 0;
 			}
+			get_all = 1;
 			break;
 		default:
 			usage(argv[0]);
@@ -72,6 +73,8 @@ int main(int argc, char **argv)
 	for (i = 0; i < len; i++) {
 		active = security_get_boolean_active(names[i]);
 		if (active < 0) {
+			if (get_all && errno == EACCES) 
+				continue;
 			fprintf(stderr, "Error getting active value for %s\n",
 				names[i]);
 			rc = -1;
@@ -84,14 +87,22 @@ int main(int argc, char **argv)
 			rc = -1;
 			goto out;
 		}
+		char *alt_name = selinux_boolean_sub(names[i]);
+		if (! alt_name) {
+			perror("Out of memory\n");
+			rc = -1;
+			goto out;
+		}
+
 		if (pending != active) {
-			printf("%s --> %s pending: %s\n", names[i],
+			printf("%s --> %s pending: %s\n", alt_name,
 			       (active ? "on" : "off"),
 			       (pending ? "on" : "off"));
 		} else {
-			printf("%s --> %s\n", names[i],
+			printf("%s --> %s\n", alt_name,
 			       (active ? "on" : "off"));
 		}
+		free(alt_name);
 	}
 
       out:
